//_________________________________________________________________________________-_______________________
// OakLibTest_Send.cpp
// 
// This is a very simple console application that demonstrates how to send data to a Oak device,
// using the Oak library provided by Toradex.
// 
// It was tested in Visual Studio 2005 running on Windows XP
//
// February 15, 2008, Peter Lischer, Toradex AG
//________________________________________________________________________________________________________
#include <windows.h>
#include <stdio.h>
#include <tchar.h>
#include "Oak.h"

#ifdef _UNICODE
#error ASCII mode required: check project properties -> Configuration Properties -> General -> Character Set
#endif

// __________________________ O a k _ P r i n t S e n s o r I n f o ______________________________________
// Print some information that is provided by the Oak Library during the initialization of the sensor
// (This routine contains no USB specific code, it just prints data)

void Oak_PrintSensorInfo(PtOakSensor Sensor)
{
	printf_s("===============================================\n");
	printf_s("%s (usr:%s)\n", Sensor->DeviceName, Sensor->UserDeviceName);
	printf_s("===============================================\n\n");
	printf_s("VID:%04X  PID:%04X  REV:%04X  SN:%s\n", Sensor->VID, Sensor->PID, Sensor->REV, Sensor->SN);
	printf_s("\nDevicePath:\n%s\n", Sensor->DevicePath);

	printf ("\n  %i Data Channels\n\n", Sensor->NumChannels);
	for (int ch=0; ch<Sensor->NumChannels; ch++)
	{
		printf_s("  ----------\n");
		printf_s("  Channel %i\n", ch);
		printf_s("  ----------\n");
		printf_s("  %s (usr:%s)\n", Sensor->Channel[ch].ChannelName, Sensor->Channel[ch].UserChannelName);
		if (Sensor->Channel[ch].IsSigned)
			printf_s("  %i Bit signed\n", Sensor->Channel[ch].BitSize);
		else
			printf_s("  %i Bit unsigned\n", Sensor->Channel[ch].BitSize);
		printf("  Resolution: 1e%i %s\n\n", Sensor->Channel[ch].UnitExponent, Sensor->Channel[ch].UnitStr);
	}
}

// __________________________ _t m a i n _________________________________________________________________
// this is already the main function.
// As you see, I tried to keep the complexity of the application low, so you can easily find what this 
// application wants to show: communication with the Oak sensor.

int _tmain(int argc, _TCHAR* argv[])
{
	bool		fSuccess;
	tOakSensor	Sensor;			// details about the Oak sensor and its data channels (name, data format, ...)
	BYTE		OutReport[33];	// USB data is transferred out reports.

	// initialization: Find one specific sensor
	// Note: this is a basic application that only reacts on one Oak Sensor, even if multiple are connected

	fSuccess = Oak_FindSensor( 0x0000,	// search for ANY PID (product ID)
							   0x0000,	// search for ANY REV (product revision)
							   NULL,	// search for ANY serial number
							   NULL,	// search for ANY device name
							   NULL,	// search for ANY channel name
							   NULL,	// search for ANY user defined device name
							   NULL,	// search for ANY user defined channel name
							   &Sensor); // <- store inforation about the first Oak sensor that matches 
										 //    the criteria above into this variable

	if (!fSuccess)	// no matching sensor found --> just leave
		return -1;

	// Just for fun: print some information about the sensor we have just found
	Oak_PrintSensorInfo(&Sensor);
	

	// The format of the OutReport depends on the type of Oak device that is connected
	// Actually, only Oak Relay and Oak IO support OutReports
	// For the description of the OutReport, please read the according datasheet of the 
	// Oak device
	switch (Sensor.PID)	//Check the type of the device
	{
		case 0x000D:	//Oak Relay recognized
			OutReport[0]= 0;	//First byte always 0
			OutReport[1]= 2;	//Command for relay 0: 0 => off; 1 => on; 2 => toggle; 3 => hold
			OutReport[2]= 2;	//Command for relay 1: 0 => off; 1 => on; 2 => toggle; 3 => hold
			OutReport[3]= 2;	//Command for relay 2: 0 => off; 1 => on; 2 => toggle; 3 => hold
			OutReport[4]= 2;	//Command for relay 3: 0 => off; 1 => on; 2 => toggle; 3 => hold

			for(int i= 0; i<10;i++)
			{
				//Send OutReport to the device
				fSuccess = Oak_SendOutReport(Sensor.DevicePath, OutReport, 5, 933);
				if (!fSuccess)
					printf_s("\n/* ERROR: Send OutReport failed */\n");
				else
				{
					printf_s("OutReport successful sent \n");
				}
				Sleep(10);
			}
		break;
		case 0x0010:	//Oak IO recognized
			OutReport[0]= 0;	//First byte always 0
			OutReport[1]= 0x00;	//OUT1CMD for I/O port A	=> Toggles all pins that are configured as output
			OutReport[2]= 0xff;	//OUT2CMD for I/O port A	=> Toggles all pins that are configured as output
			OutReport[3]= 0x00;	//OUT1CMD for I/O port B	=> Toggles all pins that are configured as output
			OutReport[4]= 0xff;	//OUT2CMD for I/O port B	=> Toggles all pins that are configured as output
			OutReport[5]= 0x00;	//OUT1CMD for I/O port C	=> Toggles all pins that are configured as output
			OutReport[6]= 0xff;	//OUT2CMD for I/O port C	=> Toggles all pins that are configured as output

			for(int i= 0; i<10;i++)
			{
				//Send OutReport to the device
				fSuccess = Oak_SendOutReport(Sensor.DevicePath, OutReport, 7, 933);
				if (!fSuccess)
					printf_s("\n/* ERROR: Send OutReport failed */\n");
				else
				{
					printf_s("OutReport successful sent \n");
				}
				Sleep(10);
			}
		break;
		default:
			printf_s("\n\n/* ERROR: Only Oak Relay and Oak IO are supported */\n");
		break;
	}

	
	// Just wait for the <ENTER> key, before closing the window
	char ch = getchar();
	return 0;
}

